package {
	import org.flixel.FlxG;
	import org.flixel.FlxGroup;
	import org.flixel.FlxPoint;
	import org.flixel.FlxSprite;
	import org.flixel.FlxText;

	public class Character extends FlxGroup {

		[Embed(source="../data/chatbubble.png")] 
		private var ImgBubble:Class;

		[Embed(source="../data/characters/fred.png")] 
		private var ImgCharacterFred:Class;

		[Embed(source="../data/characters/viki.png")] 
		private var ImgCharacterViki:Class;

		[Embed(source="../data/characters/brad.png")] 
		private var ImgCharacterBrad:Class;

		[Embed(source="../data/characters/droid.png")] 
		private var ImgCharacterDroid:Class;

		[Embed(source="../data/characters/captain.png")] 
		private var ImgCharacterCaptain:Class;

		[Embed(source="../data/characters/heather.png")] 
		private var ImgCharacterHeather:Class;

		[Embed(source="../data/characters/character_selected.png")] 
		private var ImgCharacterSelected:Class;

		[Embed(source="../data/fonts/DigitalStrip2BB_reg.TTF",fontFamily="Dialog")] 
		protected var junk:String;

		public var selected:Boolean;
		public var selectable:Boolean;
		public var characterSprite:FlxSprite;
		public var selectSprite:FlxSprite;

		private var _station:Station;
		private var _statusText:FlxText;

		public var _name:String;

		private var _isv:InternalShipView;
		private var _walking:Boolean;
		private var _midpoint:FlxPoint;
		private var _destpoint:FlxPoint;

		/*
		 * X and Y are at bottom center of the sprite
		 */
		public function Character(ISV:InternalShipView, X:Number = 0, Y:Number = 0, Name:String = null, SimpleGraphic:Class = null) {
			super();
			
			_isv = ISV;
			
			selected = false;
			selectable = true;
			_walking = false;
			_station = null;
			
			// Character Sprite
			characterSprite = new FlxSprite();
			// Character Selection
			selectSprite = new FlxSprite();
			_name = Name;
			
			if(_name == "Fred") {
				characterSprite.loadGraphic(ImgCharacterFred, false, true);
			}
			else if(_name == "Viki") {
				characterSprite.loadGraphic(ImgCharacterViki, false, true);
			}
			else if(_name == "Brad") {
				characterSprite.loadGraphic(ImgCharacterBrad, false, true);
			}
			else if(_name == "Droid") {
				characterSprite.loadGraphic(ImgCharacterDroid, false, true);
			}
			else if(_name == "Heather") {
				characterSprite.loadGraphic(ImgCharacterHeather, false, true);
			}
			else if(_name == "Captain") {
				characterSprite.loadGraphic(ImgCharacterCaptain, false, true);
			}
			else if(_name == "Droid") {
				characterSprite.loadGraphic(ImgCharacterDroid, false, true);
			} else {
				characterSprite.createGraphic(40, 100);
			}
			add(characterSprite);

			// Set properties of the group to properties of the main display item
			x = X - characterSprite.origin.x;
			y = Y - characterSprite.height;
			width = characterSprite.width;
			height = characterSprite.height;
			origin.x = characterSprite.origin.x;
			origin.y = characterSprite.origin.y;

			selectSprite.loadGraphic(ImgCharacterSelected, false, true);
			selectSprite.visible = false;
			add(selectSprite);
			
			// Status
			_statusText = new FlxText(0, 0, width);
			_statusText.text = _name;
			_statusText.size = 24;
			_statusText.alignment = "center";
			_statusText.font = "Dialog";
			_statusText.visible = false;
			add(_statusText);
			
			FlxG.mouse.show();	
		}

		public function appendSpeech(Speech:String):void {
		}

		
		override public function update():void {
			super.update();
			
			if(characterSprite.overlaps(FlxG.mouse.cursor)) {
				_statusText.visible = true;
			} else {
				_statusText.visible = false;
			}
			
			/**
			 * The control of characters depends on the 
			 * _status of the internal view
			 * _status == 1 means with control
			 **/
			if(FlxG.mouse.justPressed() && (_isv._status == 1)) {
				if(characterSprite.overlaps(FlxG.mouse.cursor)) {
					if(selectable) {
						selected = !selected;
						selectSprite.x = characterSprite.x + characterSprite.origin.x - selectSprite.origin.x;
						selectSprite.y = characterSprite.y + characterSprite.origin.y - selectSprite.origin.y;
					}
				}
				
				if(selected) {
					var isvl:uint = _isv.stations.length;
					for (var i:int = 0;i < isvl;i++) {
						if(Station(_isv.stations[i]).overlaps(FlxG.mouse.cursor) && !Station(_isv.stations[i]).occupant) {
							_station.occupant = null;
							occupyStation(_isv.stations[i]);
							selected = false;
							break;
						}
					}
				}
			}
			_statusText.x = x - 10;
			_statusText.y = y - 30;
			selectSprite.visible = selected;	
		}

		public function occupyStation(s:Station):void {
			s.occupant = this;
			_station = s;
			reset(s.x + s.origin.x - origin.x, s.y + s.origin.y - height);
		}
	}
}