package {
	import org.flixel.FlxButton;
	import org.flixel.FlxEmitter;
	import org.flixel.FlxG;
	import org.flixel.FlxGroup;
	import org.flixel.FlxSprite;

	public class InternalShipView extends FlxGroup {		
		public static const BUBBLE_FADE_INTERVAL:Number = 30;
		public static const CINEMATIC_PAUSE_INTERVAL:Number = 1;

		private static const RED:uint = 0xffff0000;
		private static const GREEN:uint = 0xff00ff00;
		private static const BLUE:uint = 0xff0000ff;
		private static const YELLOW:uint = 0xffffff00;

		[Embed(source="../data/ship_internalview.png")] 
		private var ImgBG:Class;

		[Embed(source="../data/bgtiles_big.png")] 
		private var ImgGibs:Class;

		[Embed(source="../data/engage_button.png")]
		private var ImgEngage:Class;

		[Embed(source="../data/engage_button_selected.png")]
		private var ImgEngageSelect:Class;

		/**
		 *  all the characters
		 **/
		private var _characterFred:Character;
		private var _characterViki:Character;
		private var _characterBrad:Character;
		private var _characterCaptain:Character;
		private var _characterDroid:Character;
		private var _characterHeather:Character;

		private var _cinematics:Array;
		private var _script:Script;
		private var _chatbubble:ChatBubble;

		private var _bg:FlxSprite;
		private var _bgEmitter:FlxEmitter;
		private var _engageButton:FlxButton;

		private var _bubbleFadeTimer:Number;
		private var _cinematicPauseTimer:Number;

		/**
		 * _status: 0 --> cinematics 
		 * _status: 1 --> user control
		 * _status: 2 --> after cinematics but no user control
		 **/
		public var _status:int;
		private var _controlAfterCinematic:Boolean;
		private var _processingCinematic:Boolean; // if processing one cinematic

		private var _currentCinematic:Cinematic;
		private var _currentAction:Action;

		/**
		 *  Stations
		 **/

		/*
		 * Stations on the ship
		 */
		public var stations:Array;

		
		/**
		 *  Hash Table
		 **/
		public var lookuptable:Object;

		public function InternalShipView() {
			super();
			
			_status = 0;
			_controlAfterCinematic = true;
			_processingCinematic = false;
			
			_script = new Script();
			
			// STATIONS

			stations = new Array();
			var tmpstation:Station;
			
			// navigation
			tmpstation = new Station(Script.NAV, 372, 375, YELLOW);
			tmpstation.helper = true;
			stations.push(tmpstation);
			
			tmpstation = new Station(Script.NAV, 793, 368, YELLOW);
			tmpstation.helper = true;
			stations.push(tmpstation);
			
			tmpstation = new Station(Script.NAV, 580, 406, GREEN);
			stations.push(tmpstation);
			
			// weapon
			tmpstation = new Station(Script.WEP, 274, 557, YELLOW);
			tmpstation.helper = true;
			stations.push(tmpstation);
			
			tmpstation = new Station(Script.WEP, 470, 877, YELLOW);
			tmpstation.helper = true;
			stations.push(tmpstation);
			
			tmpstation = new Station(Script.WEP, 455, 655, GREEN);
			stations.push(tmpstation);
			
			// shield
			tmpstation = new Station(Script.SHIELD, 873, 553, YELLOW);
			tmpstation.helper = true;
			stations.push(tmpstation);
			
			tmpstation = new Station(Script.SHIELD, 745, 872, YELLOW);
			tmpstation.helper = true;
			stations.push(tmpstation);
			
			tmpstation = new Station(Script.SHIELD, 744, 667, GREEN);
			stations.push(tmpstation);
			
			
			// temp
			tmpstation = new Station(Script.TEMP, 528, 530, GREEN);
			stations.push(tmpstation);
			
			tmpstation = new Station(Script.TEMP, 650, 530, GREEN);
			stations.push(tmpstation);
			
			tmpstation = new Station(Script.TEMP, 589, 587, GREEN);
			stations.push(tmpstation);
			
			// CHARACTERS
			 
			// Crew
			_characterCaptain = new Character(this, 602, 275, "Captain");
			_characterCaptain.selectable = false;
			
			_characterBrad = new Character(this, 302, 738, "Brad");
			_characterBrad.selectable = false;
						
			_characterFred = new Character(this, 898, 736, "Fred");
			_characterFred.selectable = false;
			
			// Helpers

			_characterDroid = new Character(this, 0, 0, "Droid");
			_characterDroid.dead = true;
			_characterViki = new Character(this, 0, 0, "Viki");
			_characterViki.occupyStation(Station(stations[1]));
			_characterHeather = new Character(this, 0, 0, "Heather");
			_characterHeather.occupyStation(Station(stations[6]));
			
//			_engageButton = new FlxButton(520, 940, onDeploy);
//			_engageButton.width = 220;
//			_engageButton.height = 100;
//			_engageButton.loadGraphic((new FlxSprite(0, 0, ImgEngage)), (new FlxSprite(0, 0, ImgEngageSelect)));
//			_engageButton.visible = false;
			
			_engageButton = new FlxButton(220, 440, onDeploy);
			_engageButton.width = 220;
			_engageButton.height = 100;
			_engageButton.loadGraphic((new FlxSprite(0, 0, ImgEngage)), (new FlxSprite(0, 0, ImgEngageSelect)));
			_engageButton.visible = false;
			_engageButton.scrollFactor.x = 0;
			_engageButton.scrollFactor.y = 0;

			_chatbubble = new ChatBubble(_characterFred);
			_bubbleFadeTimer = BUBBLE_FADE_INTERVAL;
			_cinematicPauseTimer = CINEMATIC_PAUSE_INTERVAL;
			_cinematics = new Array();
			
			// feed sth into _cinematics manually
			// testing only
			//queueupCinematics(_script.getCinematics());

			_bg = new FlxSprite(0, 0, ImgBG);
			
			
			// bgemitter
			_bgEmitter = new FlxEmitter(650, 650);
			_bgEmitter.gravity = 0;
			_bgEmitter.delay = 1;
			_bgEmitter.setXSpeed(-250, 0);
			_bgEmitter.setYSpeed(-300, 0);
			
			_bgEmitter.setRotation(-720, -720);
			_bgEmitter.createSprites(ImgGibs, 100, 10, true, 0);
			_bgEmitter.start(false, 1.5);
			
			lookuptable = new Object();
			lookuptable["Captain"] = Script.NAV;
			lookuptable["Brad"] = Script.WEP;
			lookuptable["Fred"] = Script.SHIELD;
			lookuptable["Viki"] = Script.NAV;
			lookuptable["Droid"] = Script.DEAD;
			lookuptable["Heather"] = Script.SHIELD;
			
			add(_bgEmitter);
			add(_bg);
			
			add(_characterCaptain);
			add(_characterViki);
			add(_characterDroid);
			add(_characterHeather);
			add(_characterFred);
			add(_characterBrad);
			add(_chatbubble);
			add(_engageButton);
		}

		public function queueupCinematics(cinematics:Array):void {
			_cinematics = cinematics;
		}

		public function playCinematics(withcontrol:Boolean = false):void {
			_status = 0; // setting the _status to cinematic
			_bubbleFadeTimer = -1;
			_controlAfterCinematic = withcontrol;
		}

		override public function update():void {
			if(FlxG.keys.justPressed("SPACE")) {
				PlayState(FlxG.state).deploy();
			}
			
			super.update();
			
			// in cinematic mode
			if(_status == 0) {
				if(!_processingCinematic) {
					_cinematicPauseTimer -= FlxG.elapsed;
					if(_cinematicPauseTimer < 0) {
						_currentCinematic = _cinematics.shift(); // fetch the next cinematic
						_processingCinematic = true;
						_cinematicPauseTimer = CINEMATIC_PAUSE_INTERVAL;
					}
				} else {
					if(_currentCinematic != null) {
						if(FlxG.mouse.justPressed())
							_bubbleFadeTimer = -1;
						else
							_bubbleFadeTimer -= FlxG.elapsed;
						if(_bubbleFadeTimer < 0) {	
							// process the current cinematic
							if((_currentAction = _currentCinematic.actions.shift()) != null) {
								trace("currentaction" + _currentAction);
								var charname:String = _currentAction.character;
								var acttype:String = _currentAction.type;
								var actcontent:String = _currentAction.content;
								
								if(charname == "F")
									_chatbubble.bindCharacter(_characterFred);
								else if(charname == "V")
									_chatbubble.bindCharacter(_characterViki);
								else if(charname == "B")
									_chatbubble.bindCharacter(_characterBrad);
								else if(charname == "C")
									_chatbubble.bindCharacter(_characterCaptain);
								else if(charname == "D")
									_chatbubble.bindCharacter(_characterDroid);
								else if(charname == "H")
									_chatbubble.bindCharacter(_characterHeather);
									
								_chatbubble.setSpeech(actcontent);
								_chatbubble.Show();
							} else {
								// set _processingcinematic to false
								// after processing the current cinematic
								// should wait about 3 seconds between the
								// two scenes	
								_processingCinematic = false;
								_chatbubble.Hide();
							}
							_bubbleFadeTimer = BUBBLE_FADE_INTERVAL;
						}
					} else {
						if(_controlAfterCinematic)
							_status = 1;
						else
							_status = 2;
						_engageButton.visible = true;
						_chatbubble.Hide();
					}
				}
			}
		}

		public function setCinematics():void {
			_status = 0;
		}

		public function setUserControl():void {
			_status = 1;
		}

		private function onDeploy():void {
			PlayState(FlxG.state).deploy();
		}

		public function getPositionTable():Object {
			lookuptable = new Object();
			
			var s:Station;
			for (var i : int = 0; i < stations.length; i++) {
				s = Station(stations[i]);
				if(s.occupant) {
					lookuptable[s.occupant._name] = s.location;
				}
			}
			
			return lookuptable;	
		}

		
		/**
		 * Triggered by PlayState to indicate a view switch
		 */
		public function switchTo():void {
			FlxG.follow(FlxG.mouse.cursor);
			FlxG.followBounds(100, -200, 1100, 1200);
			FlxG.scroll.x = -300;
			FlxG.scroll.y = -300;
			FlxG.mouse.show();
		}
	}
}