package {
	import org.flixel.FlxG;

	import flash.display.Sprite;

	[SWF(width="640", height="640", backgroundColor="#000000")]

	/*
	 * The Script class is simply a helper class for
	 * loading scripts from files
	 */

	public class Script extends Sprite {

		// Constants 
		public static const DEAD : uint = 0;
		public static const NAV : uint = 1;
		public static const WEP : uint = 2;
		public static const SHIELD : uint = 3;
		public static const TEMP : uint = 4;

		public static const CREW : Array = new Array("Captain", "Brad", "Fred");

		[Embed(source="script.txt",mimeType="application/octet-stream")] 
		private var TxtScript : Class;

		[Embed(source="story/captain.txt",mimeType="application/octet-stream")] 
		private var  CaptainScript : Class;

		[Embed(source="story/fred.txt",mimeType="application/octet-stream")] 
		private var  FredScript : Class;

		[Embed(source="story/brad.txt",mimeType="application/octet-stream")] 
		private var  BradScript : Class;

		[Embed(source="story/heather.txt",mimeType="application/octet-stream")] 
		private var  HeatherScript : Class;

		[Embed(source="story/viki.txt",mimeType="application/octet-stream")] 
		private var  VikiScript : Class;

		[Embed(source="story/droid.txt",mimeType="application/octet-stream")] 
		private var  DroidScript : Class;

		/*
		 * Cinematic timelines per character
		 */
		public var cBar : Array;
		public var fBar : Array;
		public var bBar : Array;
		public var vBar : Array;
		public var hBar : Array;
		public var dBar : Array;

		// Viki's winning path
		public const vMap : Array = new Array(NAV, NAV, WEP, WEP, WEP, WEP, WEP, WEP, WEP, WEP, WEP, WEP);

		public function Script() {
			cBar = new Array();
			fBar = new Array();
			bBar = new Array();
			vBar = new Array();
			hBar = new Array();
			dBar = new Array();
			
			// Read in script files, store cinematics in neat-o data structures
			parseScript(cBar, new CaptainScript());
			parseScript(fBar, new FredScript());
			parseScript(bBar, new BradScript());
			parseScript(vBar, new VikiScript());
			parseScript(hBar, new HeatherScript());
			parseScript(dBar, new DroidScript());
			
			
			var o:Object = new Object;
			o["Captain"] = NAV;
			o["Brad"] = WEP;
			o["Fred"] = SHIELD;
			
			o["Heather"] = SHIELD;
			o["Viki"] = NAV;
			
			getCinematics(o);
			 
		}

		// Synthetic cinematic for when Viki is smoking people out
		private function smoke() : Cinematic {
			var smoke : Cinematic = new Cinematic(0, "Smoke");
			smoke.pushAction(new Action());
			return  smoke;
		}

		// Read a script file and store the relevant info
		private function parseScript(bar : Array, contents : String) : void {
			// a blank line terminates a cinematic block
			var sceneRE : RegExp = /\[(.):(.*)\]([^\n]*)\n/;
			var lineRE : RegExp = /(.):(.*)/;
			var commentRE : RegExp = /^#.*/;
			var actionRE : RegExp = /^!.*/;
			
			contents.replace(commentRE, "");
			contents.replace(actionRE, "");
			
			var blocks : Array = contents.split(sceneRE);

			// Parse each entire cinematic block
			for (var i : uint = 1;i < blocks.length;i += 4) {
				var level : uint = uint(blocks[i]);
				var title : String = String(blocks[i + 1]);
				var options : String = String(blocks[i + 2]);
				var script : String = String(blocks[i + 3]);
			
				var lines : Array = script.split(lineRE);
				
				var cinematic : Cinematic = new Cinematic(level, title);
				
				// Parse the individual actions/lines
				for (var j : uint = 0;j < lines.length;j += 3) {
					var character : String = lines[j + 1];
					var content : String = lines[j + 2];
					
					cinematic.pushAction( new Action(character, "speak", content));
				}
				
				bar.push(cinematic);
			}
		}

		/*
		 * Input: Dictionary containing the location of each character
		 * Output: An array of cinematics relevant to this configuration
		 */
		public function getCinematics(state : Object = null) : Array {
			var cinematics : Array = new Array();

			var navRoom : Array = new Array();
			var wepRoom : Array = new Array();
			var shieldRoom : Array = new Array();

			// Order players by room
			for (var character : String in state) {
				if (state[character] == NAV) navRoom.push(character);
				if (state[character] == WEP) wepRoom.push(character);
				if (state[character] == SHIELD) shieldRoom.push(character);
			}

			var level : uint = FlxG.level;
			//level = 1;
			
			// When Brad is alone, he is lonely
			if (level < 4 && wepRoom.length <= 1) {
				cinematics.push(bBar[level]);
			}
			
			// Heather's story, sometimes with Fred
			if (state["Heather"] == WEP) {
				cinematics.push(hBar[level].updateChar("B"));
			} else if (state["Heather"] == NAV) {
				cinematics.push(hBar[level].updateChar("C"));
			} else if (state["Heather"] == SHIELD) {
				cinematics.push(fBar[level]);
			}
			
			// Viki's story, mostly with weed
			if (state["Viki"] == vMap[level]) {
				cinematics.push(vBar[level].updateChar(CREW[state["Viki"]]));
			} else if (state["Viki"] == WEP) {
				cinematics.push(smoke().updateChar("B"));
			} else if (state["Viki"] == SHIELD) {
				cinematics.push(smoke().updateChar("F"));
			} else if (state["Viki"] == NAV) {
				cinematics.push(smoke().updateChar("C"));
			}
			
			// Make each cinematic as played
			for each (var c:Cinematic in cinematics) {
				c.used = true;
				//trace(c);
				for each (var a:Action in c.actions) {
					//trace(a);
				}
			}
     
			return cinematics;
		}
	}
}