package {
	import base.Proj;

	import bosses.Boss;
	import bosses.BossComp;
	import bosses.duck.DuckProj;

	import player.BangImpact;
	import player.PlayerShip;

	import org.flixel.FlxG;
	import org.flixel.FlxGroup;
	import org.flixel.FlxObject;
	import org.flixel.FlxSprite;
	import org.flixel.FlxText;
	import org.flixel.FlxTileblock;
	import org.flixel.FlxU;

	public class SpaceView extends FlxGroup {
		// CONSTANTS
		public static const LEVEL_WIDTH:int = 800;
		public static const LEVEL_HEIGHT:int = 800;

		/*
		 * Time until level start
		 */
		private static const COUNTDOWN:Number = 1;
		/*
		 * Countdown until escape has succeeded
		 */
		private static const ESCAPE_COUNTDOWN:Number = 1;
		/*
		 * Time from defeat/victory to moving onto next level
		 */
		private static const RESULT_COUNTDOWN:Number = 1;

		// VARIABLES
		
		/*
		 * Group of Impact Objects
		 */
		public var impacts:FlxGroup;
		/*
		 * Player ship
		 */
		public var ship:PlayerShip;
		/*
		 * Player ship's group of projectiles
		 */
		public var shipProjectiles:FlxGroup;
		/*
		 * Bang visual for when the ship gets hit
		 */
		private var shipBang:BangImpact;
		/*
		 * The current boss of the level
		 */
		public var enemy:Boss;
		/*
		 * The projectiles of the current boss of the level
		 */
		public var enemyProjectiles:FlxGroup;
		/*
		 * The name of the level / boss
		 */
		public var enemyName:String;

		private var _border:FlxGroup;
		/*
		 * Shields down warning graphic
		 */
		private var _shieldsDown:FlxSprite;
		private var _shieldsDownAlphaDelta:Number;
		private var _countdownText:FlxText;
		private var _countdownTimer:Number;
		private var _resultText:FlxText;
		private var _resultTimer:Number;
		private var _nameText:FlxText;
		// Trying to escape?
		private var _escaping:Boolean;
		// Escape succeeded?
		private var _escaped:Boolean;
		// Escape Timer
		private var _escapeTimer:Number;

		// Kill Timer
		private var _killTimer:Number;
		// Kill Timer Text
		private var _killText:FlxText;

		// EMBEDDED RESOURCES

		/*
		 * Stars background
		 */
		[Embed(source='../data/bgtiles.png')]
		private const ImgBG:Class;
		
		/*
		 * Big Stars background
		 */
		[Embed(source='../data/bgtiles_big.png')]
		private const ImgBGBig:Class;

		/*
		 * Shields down warning splat
		 */
		[Embed(source='../data/shields_down_notext.png')]
		private const ImgShieldsDown:Class;

		/*
		 * Passed an array of ship upgrades from 0 to 2
		 * [ Weapons, Shields, Navigation ]
		 */
		public function SpaceView(shipUpgrades:Array, name:String):void {
			super();

			// Name
			enemyName = name;

			// Background
			initBG();
			
			// Impact group
			impacts = new FlxGroup();

			// Player Ship
			initPlayerShip(shipUpgrades);
			add(ship);
			add(shipProjectiles);
			
			// Player Ship Bang (when it gets hit)
			shipBang = new BangImpact();
			impacts.add(shipBang);

			// Boss of the level
			enemyProjectiles = new FlxGroup();
			initBoss();
			// Update once in case initial position depends on one call of the update function
			enemy.update();
			// De-Activate the boss until the countdown has started
			enemy.active = false;
			
			// Add Splat and Thud Graphics
			impacts.add(enemy.splatImagePool);
			impacts.add(enemy.thudImagePool);

			// Add Impacts
			add(impacts);

			// Level Boundary
			add(createBorder());
			
			// Shields Down Warning Graphic
			_shieldsDown = new FlxSprite(0, 0, ImgShieldsDown);
			_shieldsDown.scrollFactor.x = 0;
			_shieldsDown.scrollFactor.y = 0;
			_shieldsDown.solid = false;
			_shieldsDown.moves = false;
			_shieldsDown.visible = false;
			_shieldsDown.alpha = 0.01;
			_shieldsDownAlphaDelta = 0.01;
			add(_shieldsDown);
			
			// Countdown text
			_countdownText = new FlxText(0, FlxG.height / 2 - 50, FlxG.width, " ");
			_countdownText.scrollFactor.x = 0;
			_countdownText.scrollFactor.y = 0;
			_countdownText.alignment = "center";
			_countdownText.size = 128;
			add(_countdownText);
			
			// Name Text
			_nameText = new FlxText(0, FlxG.height / 3 - 60, FlxG.width, name);
			_nameText.scrollFactor.x = 0;
			_nameText.scrollFactor.y = 0;
			_nameText.alignment = "center";
			_nameText.size = 64;
			add(_nameText);
			
			// Start countdown timer
			_countdownTimer = COUNTDOWN;
			
			// Kill Timer
			_killTimer = 0;
			_killText = new FlxText(5, 5, FlxG.width, "Time: 0");
			_killText.scrollFactor.x = 0;
			_killText.scrollFactor.y = 0;
			_killText.size = 16;
			add(_killText);
			
			// Level result text
			_resultText = new FlxText(0, FlxG.height / 2 - 50, FlxG.width, "");
			_resultText.scrollFactor.x = 0;
			_resultText.scrollFactor.y = 0;
			_resultText.alignment = "center";
			_resultText.size = 64;
			_resultText.visible = false;
			add(_resultText);
			
			// Initialize result timer
			_resultTimer = RESULT_COUNTDOWN;
			
			// Initialize escape logic
			_escapeTimer = 0;
			_escaping = false;
			_escaped = false;
		}

		
		
		private function startFollow():void {
			FlxG.follow(ship, 5);
			FlxG.followBounds(0, 0, LEVEL_WIDTH, LEVEL_HEIGHT);
		}

		private function initBG():void {
			addStarGroup(ImgBG, 50, 1.25);
			addStarGroup(ImgBG, 50, 1.1);
			addStarGroup(ImgBGBig, 20);
		}

		/*
		 * Initialize this Level's Boss
		 */
		protected function initBoss():void {
			// No Default boss
		}

		/*
		 * Initialize the player ship
		 */
		private function initPlayerShip(shipUpgrades:Array):void {
			// Ship and object pool for bullets
			shipProjectiles = new FlxGroup();
			ship = new PlayerShip(LEVEL_WIDTH / 2, LEVEL_HEIGHT - 100, shipProjectiles, shipUpgrades);
			ship.update();
			ship.firing = false;
		}

		/*
		 * Create the Border
		 */
		private function createBorder(x:Number = 0, y:Number = 0, w:Number = LEVEL_WIDTH, h:Number = LEVEL_HEIGHT):FlxGroup {
			_border = new FlxGroup();
			_border.visible = false;
			// Ceiling
			_border.add(new FlxTileblock(x, y, w, 1));
			// Floor
			_border.add(new FlxTileblock(x, h - 1, w, 1));
			// Left
			_border.add(new FlxTileblock(x, y + 1, 1, h - 2));
			// Right
			_border.add(new FlxTileblock(w - 1, y + 1, 1, h - 2));
			return _border;
		}

		private function addStarGroup(Img:Class, e:Number = 100, sc:Number = 1):void {
			var _bg:FlxTileblock = new FlxTileblock(0 - (1 - sc)*LEVEL_WIDTH*0.5,  - (1 - sc)*LEVEL_HEIGHT*0.5, LEVEL_WIDTH * sc, LEVEL_HEIGHT * sc);
			_bg.loadGraphic(Img, e);
			_bg.solid = false;
			_bg.scrollFactor.x = sc;
			_bg.scrollFactor.y = sc;
			add(_bg);
		}

		override public function update():void {
			// Timers
			_countdownTimer -= FlxG.elapsed;
			
			// Result Timer
			if(ship.dead || enemy.dead) {
				_resultTimer -= FlxG.elapsed;
				// Result
				_resultText.visible = true;
				if(ship.dead) {
					if(_escaped)
						_resultText.text = "ESCAPED";
					else
						_resultText.text = "DEFEAT";
					ship.stopFiring();
				}
				if(enemy.dead) {
					_resultText.text = "VICTORY";
					// Only do this once, so we use ship.firing to tell if we've done this before
					if(ship.firing) {
						for each (var p : FlxObject in enemyProjectiles.members) {
							if(p.exists)
								p.exists = false;
						}
					}
					ship.stopFiring();
				}
			}
			
			// Kill Timer
			if(enemy.active && enemy.exists) {
				_killTimer += FlxG.elapsed;
				_killText.text = "Time: " + _killTimer.toFixed(2);
			}
			
			// Escape Timer
			if(_escaping) {
				_escapeTimer -= FlxG.elapsed;
				
				if(_escapeTimer < 0) {
					_escaped = true;
					ship.kill();
				}
			}
			
			// Countdown Logic (checks for enemy active to prevent extra work once we've started the level
			if(!enemy.active) {
				_countdownText.text = Math.ceil(_countdownTimer).toString();
				if(_countdownTimer < 0) {
					enemy.active = true;
					ship.startFiring();
					remove(_countdownText);
					remove(_nameText);
				}
			}
			
			super.update();
			
			//Overlap
			FlxU.overlap(ship, enemy, enemyShipOverlap);
			FlxU.overlap(ship, enemyProjectiles, enemyShipOverlap);
			FlxU.overlap(shipProjectiles, enemy, bulletEnemyOverlap);
			//Collision
			FlxU.collide(ship, _border);
			
			if(_resultTimer < 0) {
				if(ship.dead)
					PlayState(FlxG.state).defeat(_escaped);
				if(enemy.dead)
					PlayState(FlxG.state).victory();
			}
			
			// Shields Down Warning
			if(ship.health == 1) {
				_shieldsDown.alpha += _shieldsDownAlphaDelta;
				
				_shieldsDown.visible = true;
				if(_shieldsDown.alpha > 0.5) {
					_shieldsDown.alpha = 0.5;
					_shieldsDownAlphaDelta = -_shieldsDownAlphaDelta;
				}
				if(_shieldsDown.alpha <= 0) {
					_shieldsDown.alpha = 0;
					_shieldsDownAlphaDelta = -_shieldsDownAlphaDelta;
				}
			} else {
				_shieldsDown.visible = false;
			}
			
			// Escape
			if(FlxG.keys.justPressed("R")) {
				_escaping = true;
				_escapeTimer = ESCAPE_COUNTDOWN;
			}
			
			// TEMPORARY, REMOVE LATER, USED TO SKIP LEVELS
			if(FlxG.keys.justPressed("SPACE"))
				enemy.kill();
		}

		private function bulletEnemyOverlap(Obj1:FlxObject, Obj2:FlxObject):void {
			var impactGraphic:FlxObject;
			
			if(Obj1 is Proj) {
				// Get Impact Graphic
				if(BossComp(Obj2).invincible)
					impactGraphic = enemy.thudImagePool.getNextAvail();
				else
					impactGraphic = enemy.splatImagePool.getNextAvail();
				// Reset Impact Graphic to Location of Impact
				impactGraphic.reset(Obj1.x + Obj1.origin.x - impactGraphic.origin.x, Obj1.y + Obj1.origin.y - impactGraphic.origin.y);
				// Kill Projectile and deal damage				
				Obj1.kill();
				Obj2.hurt(ship.damage);
			} else {
				// Get Impact Graphic
				if(BossComp(Obj1).invincible)
					impactGraphic = enemy.thudImagePool.getNextAvail();
				else
					impactGraphic = enemy.splatImagePool.getNextAvail();
				// Reset Impact Graphic to Location of Impact
				impactGraphic.reset(Obj2.x + Obj2.origin.x - impactGraphic.origin.x, Obj2.y + Obj2.origin.y - impactGraphic.origin.y);
				// Kill Projectile and deal damage
				Obj2.kill();
				Obj1.hurt(ship.damage);
			}
		}

		private function enemyShipOverlap(Obj1:FlxObject, Obj2:FlxObject):void {
			if(Obj1 is PlayerShip) {
				if(Obj2 is Proj) {
					if(!ship.invincible)
						shipBang.reset(Obj2.x + Obj2.origin.x - shipBang.origin.x, Obj2.y + Obj2.origin.y - shipBang.origin.y);
					Obj2.kill();
				}
				Obj1.hurt(1);
			} else {
				if(Obj1 is Proj) {
					if(!ship.invincible)
						shipBang.reset(Obj1.x + Obj1.origin.x - shipBang.origin.x, Obj1.y + Obj1.origin.y - shipBang.origin.y);
					Obj1.kill();
				}
				Obj2.hurt(1);
			}
		}

		/*
		 * Triggered by PlayState to indicate a view switch
		 */
		public function switchTo():void {
			startFollow();
		}
	}
}