package base {
	import org.flixel.FlxSprite;

	/**
	 * @author Yori Kvitchko
	 */
	public class Proj extends FlxSprite {
		/*
		 * Identifies a projectile as being reflectable or not
		 */
		private var _reflectable:Boolean;
		/*
		 * Whether or not this projectile is in a normal or reflected state
		 */
		private var _reflected:Boolean;

		public function Proj() {
			super(-100, -100);
			_reflectable = false;
			_reflected = false;
			initGraphics();
		}

		
		override public function update():void {
			super.update();
			
			if(x > (SpaceView.LEVEL_WIDTH + 250) || y > (SpaceView.LEVEL_HEIGHT + 250) || x < -250 || y < -250)
				exists = false;
		}

		/*
		 * Override this function to implement initializing a graphic
		 * for the bullet
		 */
		public function initGraphics():void {
			// No default graphics
		}

		/*
		 * Override this function to implement custom send code
		 */
		public function send(X:Number, Y:Number, locationIndex:int):void {
			// Just reset the bullet to the given position
			this.reset(X - origin.x, Y - origin.y);
		}

		/*
		 * Function for telling a bullet to reflect itself
		 * if it is reflectable or kill it
		 */
		public function reflect():void {
			if(_reflectable) {
				_reflected = true;
			} else {
				kill();
			}
		}

		/*
		 * Identifies a projectile as being reflectable or not
		 */
		public function get reflectable():Boolean {  
			return _reflectable; 
		}  

		public function set reflectable(value:Boolean):void {  
			this._reflectable = value;  
		}  
	}
}
