package bosses.doomstroyer {
	import bosses.Boss;
	import bosses.BossHealthComp;

	import org.flixel.FlxG;
	import org.flixel.FlxGroup;

	/**
	 * @author Yori Kvitchko
	 */
	public class DoomBoss extends Boss {
		private static const SPAWNER_1_SWAP_INTERVAL:Number = 2.5;
		private static const SPAWNER_2_ANGLE_DELTA:Number = 50;
		private static const SWITCH_INTERVAL:Number = 5;

		private var _time:Number;
		private var _spawner_1:DoomCircleSpawner;
		private var _spawner_2:DoomCircleSpawner;

		// Used for mini-angle swap on spawner 1
		private var _swapTimer:Number;
		// Used for angle change on spawner 2
		private var _spawner2AngleDelta:Number;
		// Used to switch between spawners 1 and 2
		private var _switchInterval:Interval;

		private var _currentSpawner1:Boolean;

		public function DoomBoss(Projectiles:FlxGroup) {
			super(Projectiles, SpaceView.LEVEL_WIDTH / 2 - 250, 30, 500, 350, 650);
			
			_time = 0;
			// Spawner 1 Swap
			_swapTimer = SPAWNER_1_SWAP_INTERVAL;
			// Spawner 2 Rotate
			_spawner2AngleDelta = SPAWNER_2_ANGLE_DELTA;
			_switchInterval = new Interval([SWITCH_INTERVAL]);
			_currentSpawner1 = true;
			
			add(new DoomBossComp(Projectiles, 90, 70, this));
			add(new DoomBossTurretComp(Projectiles, 190, 240, this));
			
			// Spawner 1
			_spawner_1 = new DoomCircleSpawner(0, 0, 0.1, 90, this);
			_spawner_1.addLocation(225, 275);
			_spawner_1.addLocation(225, 275);
			_spawner_1.addLocation(275, 275);
			_spawner_1.addLocation(275, 275);
			_spawner_1.addLocation(275, 325);
			_spawner_1.addLocation(275, 325);
			_spawner_1.addLocation(225, 325);
			_spawner_1.addLocation(225, 325);
			_spawner_1.start();
			Projectiles.add(_spawner_1.projectilePool);
			add(_spawner_1);
			
			// Spawner 2
			_spawner_2 = new DoomCircleSpawner(0, 0, 0.1, 90, this);
			_spawner_2.addLocation(250, 300);
			_spawner_2.addLocation(250, 300);
			_spawner_2.addLocation(250, 300);
			_spawner_2.addLocation(250, 300);
			_spawner_2.stop();
			Projectiles.add(_spawner_2.projectilePool);
			add(_spawner_2);
			
			// Health
			add(new BossHealthComp(255, 210, this));
		}

		override public function update():void {
			_time += FlxG.elapsed;
			_switchInterval.elapse(FlxG.elapsed);
			
			if(_switchInterval.justTicked) {
				// Switch Spawners
				_currentSpawner1 = !_currentSpawner1;
				
				// Stop/Start as appropriate
				if(_currentSpawner1) {
					_spawner_1.start();
					_spawner_2.stop();
				} else {
					_spawner_1.stop();
					_spawner_2.start();
				}
			}
			
			if(_currentSpawner1) {
				// Spawner 1 Logic
				_swapTimer -= FlxG.elapsed;
				
				if(_swapTimer < 0) {
					_spawner_1.projAngle += 22.5;
					_swapTimer = SPAWNER_1_SWAP_INTERVAL;
				}
			} else {
				// Spawner 2 Logic
				_spawner_2.projAngle += _spawner2AngleDelta * FlxG.elapsed;
				if(_spawner_2.projAngle > 135) {
					_spawner_2.projAngle = 135;
					_spawner2AngleDelta = -_spawner2AngleDelta;
				}
				if(_spawner_2.projAngle < 55) {
					_spawner_2.projAngle = 55;
					_spawner2AngleDelta = -_spawner2AngleDelta;
				}
			}
			
			reset(SpaceView.LEVEL_WIDTH / 2 - 250 + Math.sin(_time) * 150, 30);
			
			super.update();
		}
	}
}
