package bosses.duck {
	import bosses.Boss;
	import bosses.BossHealthComp;

	import org.flixel.FlxG;
	import org.flixel.FlxGroup;
	import org.flixel.FlxObject;
	import org.flixel.FlxSprite;

	/**
	 * @author Yori Kvitchko
	 */
	public class DuckBoss extends Boss {
		private static const OVERDRIVE_TIME:Number = 30;
		private static const OVERDRIVE_CHARGE_TIME:Number = 3; 
		private static const BULLET_INTERVAL:Number = 1.5;
		private static const PHASE_INTERVAL:Number = 5;

		private var _core:DuckBossComp;
		private var _hb:BossHealthComp;
		private var _leftCollideGroup:FlxGroup;
		private var _rightCollideGroup:FlxGroup;

		private var _pauseInterval:Interval;
		private var _pausing:Boolean;
		private var _swapInterval:Interval;
		private var _swapping:Boolean;
		private var _xAlpha:Number;
		private var _yAlpha:Number;
		private var _xAlphaInterval:Number;
		private var _yAlphaInterval:Number;

		private var _overdriveStage:Number;
		private var _overdriveTimer:Number;

		private var _leftEyeSpawner:DuckEyeSpawner;
		private var _rightEyeSpawner:DuckEyeSpawner;
		/*
		 * Number of bullets that have been released from the rear during the swapping phase
		 */
		private var _rearBulletCounter:int;

		public function DuckBoss(Projectiles:FlxGroup) {
			super(Projectiles, SpaceView.LEVEL_WIDTH / 2 - 50, 200, 225, 256, 750);
			
			_overdriveStage = 1;
			_overdriveTimer = OVERDRIVE_TIME;
			_pauseInterval = new Interval([1]);
			_pausing = false;
			_swapInterval = new Interval([PHASE_INTERVAL, PHASE_INTERVAL]);
			_swapping = true;
			_xAlphaInterval = -1000;
			_yAlphaInterval = 200;
			_rearBulletCounter = 100; // No rear bullets the first time around
			
			// Left Eye Spawner
			_leftEyeSpawner = new DuckEyeSpawner(0, 0, BULLET_INTERVAL, true, this);
			_leftEyeSpawner.addLocation(89, 55);
			_leftEyeSpawner.addLocation(89, 55);
			_leftEyeSpawner.addLocation(89, 55);
			_leftEyeSpawner.addLocation(89, 55);
			_leftEyeSpawner.addLocation(89, 55);
			_leftEyeSpawner.addLocation(89, 55);
			_leftEyeSpawner.cycle = false;
			_leftEyeSpawner.stop();
			Projectiles.add(_leftEyeSpawner.projectilePool);
			add(_leftEyeSpawner);
			
			// Right Eye Spawner
			_rightEyeSpawner = new DuckEyeSpawner(0, 0, BULLET_INTERVAL, false, this);
			_rightEyeSpawner.addLocation(225 - 89, 55);
			_rightEyeSpawner.addLocation(225 - 89, 55);
			_rightEyeSpawner.addLocation(225 - 89, 55);
			_rightEyeSpawner.addLocation(225 - 89, 55);
			_rightEyeSpawner.addLocation(225 - 89, 55);
			_rightEyeSpawner.addLocation(225 - 89, 55);
			_rightEyeSpawner.cycle = false;
			_rightEyeSpawner.stop();
			Projectiles.add(_rightEyeSpawner.projectilePool);
			add(_rightEyeSpawner);
			_rightEyeSpawner.active = false;
			
			// Dummy colliders
			
			// * Left Collide *
			_leftCollideGroup = new FlxGroup();
			add(_leftCollideGroup);
			// Head
			_leftCollideGroup.add(new DuckBossDummyComp(projectiles, 64, 38, 59, 59, this));
			// Bill
			_leftCollideGroup.add(new DuckBossDummyComp(projectiles, 2, 86, 38, 10, this));
			_leftCollideGroup.add(new DuckBossDummyComp(projectiles, 38, 74, 32, 14, this));
			// Neck
			_leftCollideGroup.add(new DuckBossDummyComp(projectiles, 62, 94, 38, 55, this));
			// Body
			_leftCollideGroup.add(new DuckBossDummyComp(projectiles, 10, 146, 150, 80, this));
			_leftCollideGroup.add(new DuckBossDummyComp(projectiles, 26, 226, 116, 22, this));
			// Tail
			_leftCollideGroup.add(new DuckBossDummyComp(projectiles, 165, 90, 40, 105, this));
			
			// * Right Collide *
			_rightCollideGroup = new FlxGroup();
			add(_rightCollideGroup);
			// Head
			_rightCollideGroup.add(new DuckBossDummyComp(projectiles, 225 - 59 - 64, 38, 59, 59, this));
			// Bill
			_rightCollideGroup.add(new DuckBossDummyComp(projectiles, 225 - 38 - 2, 86, 38, 10, this));
			_rightCollideGroup.add(new DuckBossDummyComp(projectiles, 225 - 32 - 38, 74, 32, 14, this));
			// Neck
			_rightCollideGroup.add(new DuckBossDummyComp(projectiles, 225 - 38 - 62, 94, 38, 55, this));
			// Body
			_rightCollideGroup.add(new DuckBossDummyComp(projectiles, 225 - 150 - 10, 146, 150, 80, this));
			_rightCollideGroup.add(new DuckBossDummyComp(projectiles, 225 - 116 - 26, 226, 116, 22, this));
			// Tail
			_rightCollideGroup.add(new DuckBossDummyComp(projectiles, 225 - 40 - 165, 90, 40, 105, this));
			// Start unsolid
			setGroupSolid(_rightCollideGroup, false);
			
			// Main visual
			_core = new DuckBossComp(projectiles, 0, 0, this);
			_core.facing = FlxSprite.LEFT;
			add(_core);
			
			// Health Bar / Logo
			_hb = new BossHealthComp(45, 190, this);
			add(_hb);
		}

		override public function update():void {
			// Timers
			if(_overdriveTimer < 0 && _overdriveStage < 2) {
				_overdriveStage = 2;
				_leftEyeSpawner.interval /= _overdriveStage;
				_rightEyeSpawner.interval /= _overdriveStage;
				_overdriveTimer = OVERDRIVE_CHARGE_TIME;
				FlxG.quake.start(0.01, OVERDRIVE_CHARGE_TIME);
				_leftEyeSpawner.stop();
				_rightEyeSpawner.stop();
			} else if (_overdriveTimer < 0 && _overdriveStage < 3) {
				_overdriveStage = 3;
				_leftEyeSpawner.start();
				_rightEyeSpawner.start();
			} else {
				_overdriveTimer -= FlxG.elapsed;
			}
				
			// Swapping and pausing
			if(_pausing || _overdriveStage == 2) {
				_pauseInterval.elapse(FlxG.elapsed);
				if(_pauseInterval.justTicked)
					_pausing = false;
			} else {
				if(_swapping) {
					_xAlpha = _xAlphaInterval * FlxG.elapsed;
				
					if(x + _xAlpha < -25) {
						reset(-25, y);
						swapSides();
						_swapping = false;
						_xAlphaInterval = -_xAlphaInterval;
						_leftEyeSpawner.start();
						_rightEyeSpawner.start();
					} else if (x + _xAlpha > SpaceView.LEVEL_WIDTH - 200) {
						reset(SpaceView.LEVEL_WIDTH - 200, y);
						swapSides();
						_swapping = false;
						_xAlphaInterval = -_xAlphaInterval;
						_leftEyeSpawner.start();
						_rightEyeSpawner.start();
					} else {
						reset(x + _xAlpha, y);
					}
				
					// Custom send function to avoid using a spawner
					if(_core.facing == FlxSprite.RIGHT && Math.ceil(x / 100) > _rearBulletCounter) {
						DuckProj(_leftEyeSpawner.projectilePool.getNextAvail()).send(x + 80, y + 225, 0);
						_rearBulletCounter++;
					}
					if(_core.facing == FlxSprite.LEFT && Math.ceil((SpaceView.LEVEL_WIDTH - 225 - x) / 100) > _rearBulletCounter) {
						DuckProj(_leftEyeSpawner.projectilePool.getNextAvail()).send(x + 145, y + 225, 0);
						_rearBulletCounter++;
					}
				} else {
					_swapInterval.elapse(FlxG.elapsed);
					if(_swapInterval.justTicked) {
						_swapping = true;
						_rearBulletCounter = 0;
						_leftEyeSpawner.stop();
						_rightEyeSpawner.stop();
						_pausing = true;
					}
				
					_yAlpha = _yAlphaInterval * FlxG.elapsed * _overdriveStage;
				
					if(y + _yAlpha < 0) {
						reset(x, 0);
						_yAlphaInterval = -_yAlphaInterval;
					} else if (y + _yAlpha > SpaceView.LEVEL_HEIGHT - 256) {
						reset(x, SpaceView.LEVEL_HEIGHT - 256);
						_yAlphaInterval = -_yAlphaInterval;
					} else {
						reset(x, y + _yAlpha);
					}
				}
			}
			
			super.update();
		}

		private function swapSides():void {
			if(_core.facing == FlxSprite.LEFT) {
				// Left to Right
				_core.facing = FlxSprite.RIGHT;
				setGroupSolid(_leftCollideGroup, false);
				setGroupSolid(_rightCollideGroup, true);
				_leftEyeSpawner.active = false;
				_rightEyeSpawner.active = true;
				_hb.x += 135;
			} else {
				// Right to Left
				_core.facing = FlxSprite.LEFT;
				setGroupSolid(_leftCollideGroup, true);
				setGroupSolid(_rightCollideGroup, false);
				_leftEyeSpawner.active = true;
				_rightEyeSpawner.active = false;
				_hb.x -= 135;
			}
		}

		private function setGroupSolid(grp:FlxGroup, sol:Boolean):void {
			for each (var o : FlxObject in grp.members) {
				o.solid = sol;
			}
		}

		override public function kill():void {
			super.kill();
			// Stop in case we died during quake
			FlxG.quake.stop();
		}
	}
}
