package bosses.whip {
	import bosses.BossHealthComp;
	import base.ProjSpawner;

	import bosses.Boss;

	import org.flixel.FlxG;
	import org.flixel.FlxGroup;

	/**
	 * @author Yori Kvitchko
	 */
	public class WhipBoss extends Boss {
		private static const BULLET_INTERVAL:Number = 0.025;
		private static const BULLET_INTERVAL_2:Number = 0.1;

		private var _phaseInterval:Interval;

		private var _core:WhipCoreBossComp;

		private var _tlCorner:WhipSpawner;
		private var _trCorner:WhipSpawner;
		private var _blCorner:WhipSpawner;
		private var _brCorner:WhipSpawner;

		/*
		 * Used for one-off explosions caused when the whip "cracks"
		 */
		private var _explodeSpawners:Array;

		public function WhipBoss(Projectiles:FlxGroup) {
			super(Projectiles, SpaceView.LEVEL_WIDTH / 2, SpaceView.LEVEL_HEIGHT / 2, 100, 100, 750);
			
			_core = new WhipCoreBossComp(Projectiles, -50, -50, this);
			add(_core);
			
			// Interval for switching between phases
			_phaseInterval = new Interval([5, 5]);
			
			// Top Left
			_tlCorner = new WhipSpawner(0, 0, BULLET_INTERVAL, 225, this);
			_tlCorner.addLocation(0, 0);
			_tlCorner.cycle = false;
			_tlCorner.start();
			Projectiles.add(_tlCorner.projectilePool);
			add(_tlCorner);
			
			// Top Right
			_trCorner = new WhipSpawner(0, 0, BULLET_INTERVAL, 315, this);
			_trCorner.addLocation(0, 0);
			_trCorner.cycle = false;
			_trCorner.start();
			Projectiles.add(_trCorner.projectilePool);
			add(_trCorner);
			
			// Bottom Left
			_blCorner = new WhipSpawner(0, 0, BULLET_INTERVAL, 135, this);
			_blCorner.addLocation(0, 0);
			_blCorner.cycle = false;
			_blCorner.start();
			Projectiles.add(_blCorner.projectilePool);
			add(_blCorner);
			
			// Bottom Right
			_brCorner = new WhipSpawner(0, 0, BULLET_INTERVAL, 45, this);
			_brCorner.addLocation(0, 0);
			_brCorner.cycle = false;
			_brCorner.start();
			Projectiles.add(_brCorner.projectilePool);
			add(_brCorner);
			
			// Health Bar
			add(new BossHealthComp(0, 0, this));
			
			// Explode Spawners
			_explodeSpawners = new Array(4);
			
			var _explodeSpawner:WhipExplodeSpawner;
			
			for(var i:int = 0;i < 4;i++) {
			
				_explodeSpawner = new WhipExplodeSpawner();
				// 12 locations
				for(var j:int = 0;j < 12;j++) {
					_explodeSpawner.addLocation(0, 0);
				}
				_explodeSpawner.cycle = false;
				_explodeSpawner.stop();
				Projectiles.add(_explodeSpawner.projectilePool);
				add(_explodeSpawner);
				
				_explodeSpawners[i] = _explodeSpawner;
			}
		}

		
		override public function update():void {
			_phaseInterval.elapse(FlxG.elapsed);
			
			if(_phaseInterval.justTicked) {
				if(_phaseInterval.currentInterval) {
					_tlCorner.interval = _trCorner.interval = _blCorner.interval = _brCorner.interval = BULLET_INTERVAL_2;
				} else {
					_tlCorner.interval = _trCorner.interval = _blCorner.interval = _brCorner.interval = BULLET_INTERVAL;
				}
			}
			
			super.update();
		}

		public function triggerExplosion(X:Number, Y:Number, index:int = 0):void {
			ProjSpawner(_explodeSpawners[index]).reset(X, Y);
			ProjSpawner(_explodeSpawners[index]).trigger();
		}
	}
}
